﻿using System;
using System.Collections.Generic;

namespace QuadTreeLib
{
    public class QuadTree<T> where T : IHasRectangleF
    {
        private readonly QuadTreeNode<T> _root;

        public QuadTree(RectangleF rectangle, int maxLevels, int splitSize)
        {
            _root = new QuadTreeNode<T>(rectangle, maxLevels, splitSize);
        }

        public QuadTree(int x, int y, int width, int height, int maxLevels, int splitSize) 
            : this(new RectangleF(x, y, width, height), maxLevels, splitSize)
        { 
        }

        public int Count { get { return _root.Count; } }

        public void Insert(T item)
        {
            _root.Insert(item);
        }

        public List<T> FindIntersecting(RectangleF area)
        {
            List<T> results = new List<T>();
            return _root.FindIntersecting(area, results);
        }

        public List<T> Find(RectangleF area)
        {
            List<T> results = new List<T>();
            return _root.Find(area, results);
        }

        public void ForEach(Action<QuadTreeNode<T>> action)
        {
            _root.ForEach(action);
        }
    }
}
